@@ -36,6 +36,7 @@
 #include "bluetooth-chooser-private.h"
 #include "gnome-bluetooth-enum-types.h"
 #include "bling-spinner.h"
+#include "bluetooth-filter-widget.h"
 
 enum {
 	SELECTED_DEVICE_CHANGED,
@@ -62,8 +63,6 @@ struct _BluetoothChooserPrivate {
 	GtkCellRenderer *connected_cell;
 	GtkWidget *treeview;
 	GtkWidget *search_hbox, *search_label, *spinner;
-	GtkWidget *device_type_label, *device_type;
-	GtkWidget *device_category_label, *device_category;
 	GtkWidget *filters_vbox;
 
 	/* Current filter */
@@ -78,6 +77,7 @@ struct _BluetoothChooserPrivate {
 	guint show_device_type : 1;
 	guint show_device_category : 1;
 	guint disco_rq : 1;
+	guint internal_filter : 1;
 };
 
 G_DEFINE_TYPE(BluetoothChooser, bluetooth_chooser, GTK_TYPE_VBOX)
@@ -88,25 +88,6 @@ enum {
 	DEVICE_TYPE_FILTER_NUM_COLS
 };
 
-static const char *
-bluetooth_device_category_to_string (int type)
-{
-	switch (type) {
-	case BLUETOOTH_CATEGORY_ALL:
-		return N_("All categories");
-	case BLUETOOTH_CATEGORY_PAIRED:
-		return N_("Paired");
-	case BLUETOOTH_CATEGORY_TRUSTED:
-		return N_("Trusted");
-	case BLUETOOTH_CATEGORY_NOT_PAIRED_OR_TRUSTED:
-		return N_("Not paired or trusted");
-	case BLUETOOTH_CATEGORY_PAIRED_OR_TRUSTED:
-		return N_("Paired or trusted");
-	default:
-		return N_("Unknown");
-	}
-}
-
 static void
 bonded_to_icon (GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	      GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
@@ -359,6 +340,78 @@ bluetooth_chooser_get_selected_device_info (BluetoothChooser *self,
 	return TRUE;
 }
 
+static gboolean
+show_confirm_dialog (BluetoothChooser *chooser, const char *name)
+{
+	GtkWidget *dialog, *parent;
+	gint response;
+
+	parent = gtk_widget_get_toplevel (GTK_WIDGET (chooser));
+	dialog = gtk_message_dialog_new (GTK_WINDOW (parent), GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+					 _("Remove '%s' from the list of devices?"), name);
+	g_object_set (G_OBJECT (dialog), "secondary-text",
+		      _("If you remove the device, you will have to set it up again before next use."),
+		      NULL);
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_REMOVE, GTK_RESPONSE_ACCEPT);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	if (response == GTK_RESPONSE_ACCEPT)
+		return TRUE;
+
+	return FALSE;
+}
+
+/**
+ * bluetooth_chooser_remove_selected_device:
+ * @self: A #BluetoothChooser widget.
+ *
+ * Return value: %TRUE if the selected device was correctly removed.
+ **/
+gboolean
+bluetooth_chooser_remove_selected_device (BluetoothChooser *self)
+{
+	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
+	GtkTreeIter iter;
+	gboolean ret = FALSE;
+	GError *err = NULL;
+	gchar *name;
+	DBusGProxy *device, *adapter;
+
+	if (gtk_tree_selection_get_selected (priv->selection, NULL, &iter) == FALSE)
+		return FALSE;
+
+	gtk_tree_model_get (priv->filter, &iter,
+			    BLUETOOTH_COLUMN_PROXY, &device,
+			    BLUETOOTH_COLUMN_ALIAS, &name, -1);
+
+	adapter = bluetooth_client_get_default_adapter(priv->client);
+
+	if (show_confirm_dialog (self, name) != FALSE) {
+		const gchar *device_path;
+
+		device_path = dbus_g_proxy_get_path (device);
+
+		if (dbus_g_proxy_call (adapter, "RemoveDevice", &err,
+				       DBUS_TYPE_G_OBJECT_PATH, device_path,
+				       G_TYPE_INVALID, G_TYPE_INVALID) == FALSE) {
+			g_warning ("Failed to remove device %s: %s", name,
+				   err->message);
+			g_error_free (err);
+		} else {
+			ret = TRUE;
+		}
+		g_object_unref (adapter);
+	}
+
+	return ret;
+}
+
 /**
  * bluetooth_chooser_get_model:
  * @self: A BluetoothChooser widget.
@@ -369,22 +422,36 @@ GtkTreeModel *
 bluetooth_chooser_get_model (BluetoothChooser *self)
 {
 	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-	
+
 	return priv->model;
 }
 
 /**
- * bluetooth_chooser_get_device_column:
+ * bluetooth_chooser_get_type_column:
  * @self: A BluetoothChooser widget.
  *
- * Return value: A GtkTreeViewColumn pointer to the device column of the BluetoothChooser.
+ * Return value: A GtkTreeViewColumn pointer to the type column of the BluetoothChooser.
  **/
 GtkTreeViewColumn *
-bluetooth_chooser_get_device_column (BluetoothChooser *self)
+bluetooth_chooser_get_type_column (BluetoothChooser *self)
 {
 	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-	
-	return gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), 0);
+
+	return gtk_tree_view_get_column (GTK_TREE_VIEW (priv->treeview), 1);
+}
+
+/**
+ * bluetooth_chooser_get_treeview:
+ * @self: A BluetoothChooser widget.
+ *
+ * Return value: The #GtkTreeView pointer for the #BluetoothChooser.
+ **/
+GtkWidget *
+bluetooth_chooser_get_treeview (BluetoothChooser *self)
+{
+	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
+
+	return priv->treeview;
 }
 
 /**
@@ -528,77 +595,21 @@ filter_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 }
 
 static void
-filter_type_changed_cb (GtkComboBox *widget, gpointer data)
+filter_type_changed_cb (GObject *object, GParamSpec *spec, gpointer user_data)
 {
-	BluetoothChooser *self = BLUETOOTH_CHOOSER (data);
-	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-	GtkTreeIter iter;
-
-	if (gtk_combo_box_get_active_iter (widget, &iter) == FALSE)
-		return;
-
-	gtk_tree_model_get (GTK_TREE_MODEL (priv->device_type_filter_model), &iter,
-			    DEVICE_TYPE_FILTER_COL_MASK, &(priv->device_type_filter),
-			    -1);
+	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(object);
 
 	if (priv->filter)
 		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->filter));
-	g_object_notify (G_OBJECT(self), "device-type-filter");
 }
 
 static void
-set_combobox_from_filter (BluetoothChooser *self)
+filter_category_changed_cb (GObject *object, GParamSpec *spec, gpointer user_data)
 {
-	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-	GtkTreeIter iter;
-	gboolean cont;
+	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(object);
 
-	/* Look for an exact matching filter first */
-	cont = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->device_type_filter_model),
-					      &iter);
-	while (cont != FALSE) {
-		int mask;
-
-		gtk_tree_model_get (GTK_TREE_MODEL (priv->device_type_filter_model), &iter,
-				    DEVICE_TYPE_FILTER_COL_MASK, &mask, -1);
-		if (mask == priv->device_type_filter) {
-			gtk_combo_box_set_active_iter (GTK_COMBO_BOX(priv->device_type), &iter);
-			return;
-		}
-		cont = gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->device_type_filter_model), &iter);
-	}
-
-	/* Then a fuzzy match */
-	cont = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->device_type_filter_model),
-					      &iter);
-	while (cont != FALSE) {
-		int mask;
-
-		gtk_tree_model_get (GTK_TREE_MODEL (priv->device_type_filter_model), &iter,
-				    DEVICE_TYPE_FILTER_COL_MASK, &mask,
-				    -1);
-		if (mask & priv->device_type_filter) {
-			gtk_combo_box_set_active_iter (GTK_COMBO_BOX(priv->device_type), &iter);
-			return;
-		}
-		cont = gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->device_type_filter_model), &iter);
-	}
-
-	/* Then just set the any then */
-	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->device_type_filter_model), &iter);
-	gtk_combo_box_set_active_iter (GTK_COMBO_BOX(priv->device_type), &iter);
-}
-
-static void
-filter_category_changed_cb (GtkComboBox *widget, gpointer data)
-{
-	BluetoothChooser *self = BLUETOOTH_CHOOSER (data);
-	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-
-	priv->device_category_filter = gtk_combo_box_get_active (GTK_COMBO_BOX(priv->device_category));
 	if (priv->filter)
 		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->filter));
-	g_object_notify (G_OBJECT(self), "device-category-filter");
 }
 
 static void
@@ -776,23 +787,14 @@ static void
 bluetooth_chooser_init(BluetoothChooser *self)
 {
 	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
-	char *str;
-	int i;
 
 	GtkWidget *vbox;
-	GtkWidget *label;
 	GtkWidget *alignment;
 	GtkWidget *hbox;
 	GtkWidget *scrolled_window;
-	GtkWidget *table;
-	GtkCellRenderer *renderer;
 
 	gtk_widget_push_composite_child ();
 
-	priv->show_paired = FALSE;
-	priv->show_searching = FALSE;
-	priv->device_type_filter = BLUETOOTH_TYPE_ANY;
-
 	priv->client = bluetooth_client_new ();
 
 	/* Setup the widget itself */
@@ -851,116 +853,13 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_IN);
 
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (self), vbox, FALSE, TRUE, 0);
-	priv->filters_vbox = vbox;
-	gtk_widget_set_no_show_all (priv->filters_vbox, TRUE);
-
 	/* The filters */
-	str = g_strdup_printf ("<b>%s</b>", _("Show Only Bluetooth Devices With..."));
-	label = gtk_label_new (str);
-	g_free (str);
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-
-	alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
-	gtk_widget_show (alignment);
-	gtk_box_pack_start (GTK_BOX (vbox), alignment, TRUE, TRUE, 0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
-
-	table = gtk_table_new (2, 2, FALSE);
-	gtk_widget_show (table);
-	gtk_container_add (GTK_CONTAINER (alignment), table);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-
-	/* The device category filter */
-	label = gtk_label_new_with_mnemonic (_("Device _category:"));
-	gtk_widget_set_no_show_all (label, TRUE);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	priv->device_category_label = label;
-
-	priv->device_category = gtk_combo_box_new_text ();
-	gtk_widget_set_no_show_all (priv->device_category, TRUE);
-	gtk_widget_show (priv->device_category);
-	gtk_table_attach (GTK_TABLE (table), priv->device_category, 1, 2, 1, 2,
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_widget_set_tooltip_text (priv->device_category, _("Select the device category to filter above list"));
-	for (i = 0; i < BLUETOOTH_CATEGORY_NUM_CATEGORIES; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX(priv->device_category),
-					   _(bluetooth_device_category_to_string (i)));
-	}
-	g_signal_connect (G_OBJECT (priv->device_category), "changed",
-			  G_CALLBACK(filter_category_changed_cb), self);
-	gtk_combo_box_set_active (GTK_COMBO_BOX(priv->device_category), priv->device_category_filter);
-	if (priv->show_device_category) {
-		gtk_widget_show (priv->device_category_label);
-		gtk_widget_show (priv->device_category);
-	}
-
-	/* The device type filter */
-	label = gtk_label_new_with_mnemonic (_("Device _type:"));
-	gtk_widget_set_no_show_all (label, TRUE);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	priv->device_type_label = label;
-
 	priv->device_type_filter_model = GTK_TREE_MODEL (gtk_list_store_new (DEVICE_TYPE_FILTER_NUM_COLS,
-									     G_TYPE_STRING, G_TYPE_INT));
-	priv->device_type = gtk_combo_box_new_with_model (priv->device_type_filter_model);
-	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->device_type), renderer, TRUE);
-	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->device_type), renderer, "text", DEVICE_TYPE_FILTER_COL_NAME);
-
-	gtk_widget_set_no_show_all (priv->device_type, TRUE);
-	gtk_widget_show (priv->device_type);
-	gtk_table_attach (GTK_TABLE (table), priv->device_type, 1, 2, 0, 1,
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_widget_set_tooltip_text (priv->device_type, _("Select the device type to filter above list"));
-	gtk_list_store_insert_with_values (GTK_LIST_STORE (priv->device_type_filter_model), NULL, G_MAXUINT32,
-					   DEVICE_TYPE_FILTER_COL_NAME, _(bluetooth_type_to_string (BLUETOOTH_TYPE_ANY)),
-					   DEVICE_TYPE_FILTER_COL_MASK, BLUETOOTH_TYPE_ANY,
-					   -1);
-	gtk_list_store_insert_with_values (GTK_LIST_STORE (priv->device_type_filter_model), NULL, G_MAXUINT32,
-					   DEVICE_TYPE_FILTER_COL_NAME, _("Input devices (mice, keyboards, ...)"),
-					   DEVICE_TYPE_FILTER_COL_MASK, BLUETOOTH_TYPE_INPUT,
-					   -1);
-	gtk_list_store_insert_with_values (GTK_LIST_STORE (priv->device_type_filter_model), NULL, G_MAXUINT32,
-					   DEVICE_TYPE_FILTER_COL_NAME, _("Headphones, headsets and other audio devices"),
-					   DEVICE_TYPE_FILTER_COL_MASK, BLUETOOTH_TYPE_AUDIO,
-					   -1);
-	/* The types match the types used in bluetooth-client.h */
-	for (i = 1; i < _BLUETOOTH_TYPE_NUM_TYPES; i++) {
-		int mask = 1 << i;
-		if (mask & BLUETOOTH_TYPE_INPUT || mask & BLUETOOTH_TYPE_AUDIO)
-			continue;
-		gtk_list_store_insert_with_values (GTK_LIST_STORE (priv->device_type_filter_model), NULL, G_MAXUINT32,
-						   DEVICE_TYPE_FILTER_COL_NAME, _(bluetooth_type_to_string (mask)),
-						   DEVICE_TYPE_FILTER_COL_MASK, mask,
-						   -1);
-	}
-	g_signal_connect (G_OBJECT (priv->device_type), "changed",
-			  G_CALLBACK(filter_type_changed_cb), self);
-	set_combobox_from_filter (self);
-	if (priv->show_device_type) {
-		gtk_widget_show (priv->device_type_label);
-		gtk_widget_show (priv->device_type);
-	}
-
-	/* The services filter */
-	priv->device_service_filter = NULL;
+						         G_TYPE_STRING, G_TYPE_INT));
+	priv->filters_vbox = bluetooth_filter_widget_new ();
+	gtk_widget_show (priv->filters_vbox);
+	gtk_box_pack_start (GTK_BOX (self), priv->filters_vbox, FALSE, TRUE, 0);
+	gtk_widget_set_no_show_all (priv->filters_vbox, TRUE);
 
 	/* if filters are not visible hide the vbox */
 	if (!priv->show_device_type && !priv->show_device_category)
@@ -969,9 +868,35 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	priv->default_adapter_changed_id = g_signal_connect (priv->client, "notify::default-adapter",
 							     G_CALLBACK (default_adapter_changed), self);
 
+	g_signal_connect(self, "notify::device-type-filter",
+			 G_CALLBACK(filter_type_changed_cb), NULL);
+	g_signal_connect(self, "notify::device-category-filter",
+			 G_CALLBACK(filter_category_changed_cb), NULL);
+
 	gtk_widget_pop_composite_child ();
 }
 
+static GObject *
+bluetooth_chooser_constructor (GType                  type,
+			       guint                  n_construct_params,
+			       GObjectConstructParam *construct_params)
+{
+	BluetoothChooser *self;
+	BluetoothChooserPrivate *priv;
+	GObject *object;
+
+	object = G_OBJECT_CLASS (bluetooth_chooser_parent_class)->constructor (type,
+									       n_construct_params,
+									       construct_params);
+	self = BLUETOOTH_CHOOSER (object);
+	priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
+
+	if (priv->internal_filter) {
+		bluetooth_filter_widget_bind_filter (BLUETOOTH_FILTER_WIDGET (priv->filters_vbox), self);
+	}
+	return object;
+}
+
 static void
 bluetooth_chooser_finalize (GObject *object)
 {
@@ -1009,7 +934,8 @@ enum {
 	PROP_SHOW_DEVICE_CATEGORY,
 	PROP_DEVICE_TYPE_FILTER,
 	PROP_DEVICE_CATEGORY_FILTER,
-	PROP_DEVICE_SERVICE_FILTER
+	PROP_DEVICE_SERVICE_FILTER,
+	PROP_INTERNAL_FILTER
 };
 
 static void
@@ -1038,29 +964,29 @@ bluetooth_chooser_set_property (GObject *object, guint prop_id,
 		break;
 	case PROP_SHOW_DEVICE_TYPE:
 		priv->show_device_type = g_value_get_boolean (value);
-		g_object_set (G_OBJECT (priv->device_type_label), "visible", priv->show_device_type, NULL);
-		g_object_set (G_OBJECT (priv->device_type), "visible", priv->show_device_type, NULL);
-		if (priv->show_device_type || priv->show_device_category)
-			g_object_set (G_OBJECT (priv->filters_vbox), "visible", TRUE, NULL);
-		else
-			g_object_set (G_OBJECT (priv->filters_vbox), "visible", FALSE, NULL);
+		if (priv->internal_filter) {
+			if (priv->show_device_type || priv->show_device_category)
+				g_object_set (G_OBJECT (priv->filters_vbox), "visible", TRUE, NULL);
+			else
+				g_object_set (G_OBJECT (priv->filters_vbox), "visible", FALSE, NULL);
+		}
 		break;
 	case PROP_SHOW_DEVICE_CATEGORY:
 		priv->show_device_category = g_value_get_boolean (value);
-		g_object_set (G_OBJECT (priv->device_category_label), "visible", priv->show_device_category, NULL);
-		g_object_set (G_OBJECT (priv->device_category), "visible", priv->show_device_category, NULL);
-		if (priv->show_device_type || priv->show_device_category)
-			g_object_set (G_OBJECT (priv->filters_vbox), "visible", TRUE, NULL);
-		else
-			g_object_set (G_OBJECT (priv->filters_vbox), "visible", FALSE, NULL);
+		if (priv->internal_filter) {
+			if (priv->show_device_type || priv->show_device_category)
+				g_object_set (G_OBJECT (priv->filters_vbox), "visible", TRUE, NULL);
+			else
+				g_object_set (G_OBJECT (priv->filters_vbox), "visible", FALSE, NULL);
+		}
 		break;
 	case PROP_DEVICE_TYPE_FILTER:
 		priv->device_type_filter = g_value_get_int (value);
-		set_combobox_from_filter (BLUETOOTH_CHOOSER (object));
+		g_object_notify (object, "device-type-filter");
 		break;
 	case PROP_DEVICE_CATEGORY_FILTER:
 		priv->device_category_filter = g_value_get_enum (value);
-		gtk_combo_box_set_active (GTK_COMBO_BOX(priv->device_category), priv->device_category_filter);
+		g_object_notify (object, "device-category-filter");
 		break;
 	case PROP_DEVICE_SERVICE_FILTER:
 		g_free (priv->device_service_filter);
@@ -1068,6 +994,10 @@ bluetooth_chooser_set_property (GObject *object, guint prop_id,
 		if (priv->filter)
 			gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->filter));
 		break;
+	case PROP_INTERNAL_FILTER:
+		priv->internal_filter = g_value_get_boolean (value);
+		g_object_set (G_OBJECT (priv->filters_vbox), "visible", priv->internal_filter, NULL);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -1109,6 +1039,9 @@ bluetooth_chooser_get_property (GObject *object, guint prop_id,
 	case PROP_DEVICE_SERVICE_FILTER:
 		g_value_set_string (value, priv->device_service_filter);
 		break;
+	case PROP_INTERNAL_FILTER:
+		g_value_set_boolean (value, priv->internal_filter);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		break;
@@ -1118,7 +1051,7 @@ bluetooth_chooser_get_property (GObject *object, guint prop_id,
 static void
 bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 {
-	/* Use to calculate the maximum value for the 
+	/* Use to calculate the maximum value for the
 	 * device-type-filter value */
 	guint i;
 	int max_filter_val;
@@ -1127,6 +1060,7 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 
 	g_type_class_add_private(klass, sizeof(BluetoothChooserPrivate));
 
+	G_OBJECT_CLASS(klass)->constructor = bluetooth_chooser_constructor;
 	G_OBJECT_CLASS(klass)->finalize = bluetooth_chooser_finalize;
 
 	G_OBJECT_CLASS(klass)->set_property = bluetooth_chooser_set_property;
@@ -1168,21 +1102,24 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 										    FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_DEVICE_TYPE, g_param_spec_boolean ("show-device-type",
-										      "show-device-type", "Whether to show the device type filter", TRUE, G_PARAM_READWRITE));
+										      "show-device-type", "Whether to show the device type filter", TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_SHOW_DEVICE_CATEGORY, g_param_spec_boolean ("show-device-category",
-											  "show-device-category", "Whether to show the device category filter", TRUE, G_PARAM_READWRITE));
+											  "show-device-category", "Whether to show the device category filter", TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	for (i = 0, max_filter_val = 0 ; i < _BLUETOOTH_TYPE_NUM_TYPES; i++)
 		max_filter_val += 1 << i;
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_DEVICE_TYPE_FILTER, g_param_spec_int ("device-type-filter",
-					 					    "device-type-filter", "A bitmask of #BluetoothType to show", BLUETOOTH_TYPE_ANY, max_filter_val, BLUETOOTH_TYPE_ANY, G_PARAM_READWRITE));
+										    "device-type-filter", "A bitmask of #BluetoothType to show", BLUETOOTH_TYPE_ANY, max_filter_val, 1, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_DEVICE_CATEGORY_FILTER, g_param_spec_enum ("device-category-filter",
-											 "device-category-filter", "The #BluetoothCategory to show", BLUETOOTH_TYPE_CATEGORY, BLUETOOTH_CATEGORY_ALL, G_PARAM_READWRITE));
+											 "device-category-filter", "The #BluetoothCategory to show", BLUETOOTH_TYPE_CATEGORY, BLUETOOTH_CATEGORY_ALL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_DEVICE_SERVICE_FILTER, g_param_spec_string ("device-service-filter",
 											  "device-service-filter", "A string representing the service to filter for", NULL, G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS(klass),
+					 PROP_INTERNAL_FILTER, g_param_spec_boolean ("has-internal-device-filter",
+											  "has-internal-device-filter", "Whether the #BluetoothChooser should be constructed with a visible #BluetoothFilterWidget", TRUE, G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 }
 
 /**
