@@ -102,7 +102,13 @@ enum {
 
 G_DEFINE_TYPE(BluetoothClient, bluetooth_client, G_TYPE_OBJECT)
 
-const gchar *bluetooth_type_to_string(guint type)
+/**
+ * bluetooth_type_to_string:
+ * @type: a #BluetoothType
+ *
+ * Return value: the string representation of the @type passed. Do not free the return value.
+ **/
+const gchar *bluetooth_type_to_string(BluetoothType type)
 {
 	switch (type) {
 	case BLUETOOTH_TYPE_ANY:
@@ -139,6 +145,12 @@ const gchar *bluetooth_type_to_string(guint type)
 	}
 }
 
+/**
+ * bluetooth_verify_address:
+ * @bdaddr: a string representing a Bluetooth address
+ *
+ * Return value: Whether the string is a valid Bluetooth address. This does not contact the device in any way.
+ **/
 gboolean
 bluetooth_verify_address (const char *bdaddr)
 {
@@ -508,6 +520,12 @@ uuid16_to_string (guint uuid16, const char *uuid)
 	}
 }
 
+/**
+ * bluetooth_uuid_to_string:
+ * @uuid: a string representing a Bluetooth UUID
+ *
+ * Return value: a string representing a human-readable (but not usable for display to users) version of the @uuid. Do not free the return value.
+ **/
 const char *
 bluetooth_uuid_to_string (const char *uuid)
 {
@@ -638,7 +656,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 	gboolean paired, trusted, connected;
 	int legacypairing;
 	guint type;
-	gint rssi;
 	GtkTreeIter iter;
 	gboolean cont;
 
@@ -669,9 +686,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 		value = g_hash_table_lookup(hash, "Icon");
 		icon = value ? g_value_get_string(value) : "bluetooth";
 
-		value = g_hash_table_lookup(hash, "RSSI");
-		rssi = value ? g_value_get_int(value) : 0;
-
 		value = g_hash_table_lookup(hash, "Paired");
 		paired = value ? g_value_get_boolean(value) : FALSE;
 
@@ -708,10 +722,13 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 					BLUETOOTH_COLUMN_NAME, name,
 					BLUETOOTH_COLUMN_TYPE, type,
 					BLUETOOTH_COLUMN_ICON, icon,
-					BLUETOOTH_COLUMN_RSSI, rssi,
-					BLUETOOTH_COLUMN_UUIDS, uuids,
 					BLUETOOTH_COLUMN_LEGACYPAIRING, legacypairing,
 					-1);
+			if (uuids != NULL) {
+				gtk_tree_store_set(priv->store, &iter,
+						   BLUETOOTH_COLUMN_UUIDS, uuids,
+						   -1);
+			}
 
 			if (device != NULL) {
 				services = device_list_nodes (device, client, FALSE);
@@ -741,7 +758,6 @@ static void add_device(DBusGProxy *adapter, GtkTreeIter *parent,
 				BLUETOOTH_COLUMN_NAME, name,
 				BLUETOOTH_COLUMN_TYPE, type,
 				BLUETOOTH_COLUMN_ICON, icon,
-				BLUETOOTH_COLUMN_RSSI, rssi,
 				BLUETOOTH_COLUMN_PAIRED, paired,
 				BLUETOOTH_COLUMN_TRUSTED, trusted,
 				BLUETOOTH_COLUMN_CONNECTED, connected,
@@ -838,6 +854,12 @@ static void adapter_changed(DBusGProxy *adapter, const char *property,
 			priv->default_adapter_powered = powered;
 			g_object_notify (G_OBJECT (client), "default-adapter-powered");
 		}
+	} else if (g_str_equal(property, "Discoverable") == TRUE) {
+		gboolean discoverable = g_value_get_boolean(value);
+
+		gtk_tree_store_set(priv->store, &iter,
+				BLUETOOTH_COLUMN_DISCOVERABLE, discoverable, -1);
+		notify = TRUE;
 	}
 
 	if (notify != FALSE) {
@@ -861,7 +883,7 @@ static void adapter_added(DBusGProxy *manager,
 	GHashTable *hash = NULL;
 	GValue *value;
 	const gchar *address, *name;
-	gboolean discovering, powered;
+	gboolean discovering, discoverable, powered;
 
 	DBG("client %p path %s", client, path);
 
@@ -884,10 +906,14 @@ static void adapter_added(DBusGProxy *manager,
 
 		value = g_hash_table_lookup(hash, "Devices");
 		devices = value ? g_value_get_boxed (value) : NULL;
+
+		value = g_hash_table_lookup (hash, "Discoverable");
+		discoverable = value ? g_value_get_boolean (value) : FALSE;
 	} else {
 		address = NULL;
 		name = NULL;
 		discovering = FALSE;
+		discoverable = FALSE;
 		powered = FALSE;
 		devices = NULL;
 	}
@@ -897,6 +923,7 @@ static void adapter_added(DBusGProxy *manager,
 					  BLUETOOTH_COLUMN_ADDRESS, address,
 					  BLUETOOTH_COLUMN_NAME, name,
 					  BLUETOOTH_COLUMN_DISCOVERING, discovering,
+					  BLUETOOTH_COLUMN_DISCOVERABLE, discoverable,
 					  BLUETOOTH_COLUMN_POWERED, powered,
 					  -1);
 
@@ -1064,9 +1091,9 @@ static void bluetooth_client_init(BluetoothClient *client)
 
 	priv->store = gtk_tree_store_new(_BLUETOOTH_NUM_COLUMNS, G_TYPE_OBJECT,
 					 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-					 G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INT,
+					 G_TYPE_UINT, G_TYPE_STRING,
 					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
-					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_INT,
+					 G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_INT,
 					 G_TYPE_BOOLEAN, G_TYPE_HASH_TABLE, G_TYPE_STRV);
 
 	priv->dbus = dbus_g_proxy_new_for_name(connection, DBUS_SERVICE_DBUS,
@@ -1194,6 +1221,11 @@ static void bluetooth_client_class_init(BluetoothClientClass *klass)
 	}
 }
 
+/**
+ * bluetooth_client_new:
+ *
+ * Return value: a reference to the #BluetoothClient singleton. Unref the object when done.
+ **/
 BluetoothClient *bluetooth_client_new(void)
 {
 	static BluetoothClient *bluetooth_client = NULL;
@@ -1210,6 +1242,12 @@ BluetoothClient *bluetooth_client_new(void)
 	return bluetooth_client;
 }
 
+/**
+ * bluetooth_client_get_model:
+ * @client: a #BluetoothClient object
+ *
+ * Return value: an unfiltered #GtkTreeModel representing the adapter and devices available on the system.
+ **/
 GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
 {
 	BluetoothClientPrivate *priv;
@@ -1225,6 +1263,15 @@ GtkTreeModel *bluetooth_client_get_model (BluetoothClient *client)
 	return model;
 }
 
+/**
+ * bluetooth_client_get_filter_model:
+ * @client: a #BluetoothClient object
+ * @func: a #GtkTreeModelFilterVisibleFunc
+ * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
+ * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
+ *
+ * Return value: a #GtkTreeModel of devices filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ **/
 GtkTreeModel *bluetooth_client_get_filter_model (BluetoothClient *client,
 						 GtkTreeModelFilterVisibleFunc func,
 						 gpointer data, GDestroyNotify destroy)
@@ -1264,6 +1311,12 @@ static gboolean adapter_filter(GtkTreeModel *model,
 	return active;
 }
 
+/**
+ * bluetooth_client_get_adapter_model:
+ * @client: a #BluetoothClient object
+ *
+ * Return value: a filtered #GtkTreeModel with only adapters present.
+ **/
 GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
 {
 	DBG("client %p", client);
@@ -1272,6 +1325,13 @@ GtkTreeModel *bluetooth_client_get_adapter_model (BluetoothClient *client)
 						  NULL, NULL);
 }
 
+/**
+ * bluetooth_client_get_device_model:
+ * @client: a #BluetoothClient object
+ * @adapter: a #DBusGProxy of the adapter object, or %NULL to get the default adapter.
+ *
+ * Return value: a filtered #GtkTreeModel with only devices belonging to the selected adapter listed. Note that the model will follow a specific adapter, and will not follow the default-adapter when %NULL is passed.
+ **/
 GtkTreeModel *bluetooth_client_get_device_model (BluetoothClient *client,
 						 DBusGProxy *adapter)
 {
@@ -1324,6 +1384,15 @@ GtkTreeModel *bluetooth_client_get_device_model (BluetoothClient *client,
 	return model;
 }
 
+/**
+ * bluetooth_client_get_device_filter_model:
+ * @client: a #BluetoothClient object
+ * @func: a #GtkTreeModelFilterVisibleFunc
+ * @data: user data to pass to gtk_tree_model_filter_set_visible_func()
+ * @destroy: a destroy function for gtk_tree_model_filter_set_visible_func()
+ *
+ * Return value: a #GtkTreeModel of adapters filtered using the @func, @data and @destroy arguments to pass to gtk_tree_model_filter_set_visible_func().
+ **/
 GtkTreeModel *bluetooth_client_get_device_filter_model(BluetoothClient *client,
 		DBusGProxy *adapter, GtkTreeModelFilterVisibleFunc func,
 				gpointer data, GDestroyNotify destroy)
@@ -1342,6 +1411,12 @@ GtkTreeModel *bluetooth_client_get_device_filter_model(BluetoothClient *client,
 	return model;
 }
 
+/**
+ * bluetooth_client_get_default_adapter:
+ * @client: a #BluetoothClient object
+ *
+ * Return value: a #DBusGProxy object representing the default adapter, or %NULL if no adapters are present.
+ **/
 DBusGProxy *bluetooth_client_get_default_adapter(BluetoothClient *client)
 {
 	BluetoothClientPrivate *priv = BLUETOOTH_CLIENT_GET_PRIVATE(client);
@@ -1375,6 +1450,14 @@ DBusGProxy *bluetooth_client_get_default_adapter(BluetoothClient *client)
 	return NULL;
 }
 
+/**
+ * bluetooth_client_start_discovery:
+ * @client: a #BluetoothClient object
+ *
+ * Start a discovery on the default adapter.
+ *
+ * Return value: Whether a discovery was successfully started on the default adapter.
+ **/
 gboolean bluetooth_client_start_discovery(BluetoothClient *client)
 {
 	DBusGProxy *adapter;
@@ -1394,6 +1477,14 @@ gboolean bluetooth_client_start_discovery(BluetoothClient *client)
 	return TRUE;
 }
 
+/**
+ * bluetooth_client_stop_discovery:
+ * @client: a #BluetoothClient object
+ *
+ * Stop a discovery started on the default adapter.
+ *
+ * Return value: Whether a discovery was successfully stopped on the default adapter.
+ **/
 gboolean bluetooth_client_stop_discovery(BluetoothClient *client)
 {
 	DBusGProxy *adapter;
@@ -1413,6 +1504,59 @@ gboolean bluetooth_client_stop_discovery(BluetoothClient *client)
 	return TRUE;
 }
 
+/**
+ * bluetooth_client_set_discoverable:
+ * @client: a #BluetoothClient object
+ * @discoverable: whether the device should be discoverable
+ *
+ * Sets the default adapter's discoverable status.
+ *
+ * Return value: Whether setting the state on the default adapter was successful.
+ **/
+gboolean
+bluetooth_client_set_discoverable (BluetoothClient *client,
+				   gboolean discoverable)
+{
+	DBusGProxy *adapter;
+	GValue disco = { 0 };
+	GValue timeout = { 0 };
+	gboolean ret;
+
+	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
+
+	DBG("client %p", client);
+
+	adapter = bluetooth_client_get_default_adapter (client);
+	if (adapter == NULL)
+		return FALSE;
+
+	g_value_init (&disco, G_TYPE_BOOLEAN);
+	g_value_init (&timeout, G_TYPE_UINT);
+
+	g_value_set_boolean (&disco, discoverable);
+	g_value_set_uint (&timeout, 0);
+
+	ret = dbus_g_proxy_call (adapter, "SetProperty", NULL,
+				 G_TYPE_STRING, "Discoverable",
+				 G_TYPE_VALUE, &disco,
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+	if (ret == FALSE)
+		goto bail;
+
+	ret = dbus_g_proxy_call (adapter, "SetProperty", NULL,
+				 G_TYPE_STRING, "DiscoverableTimeout",
+				 G_TYPE_VALUE, &timeout,
+				 G_TYPE_INVALID, G_TYPE_INVALID);
+
+bail:
+	g_value_unset (&disco);
+	g_value_unset (&timeout);
+
+	g_object_unref(adapter);
+
+	return ret;
+}
+
 typedef struct {
 	BluetoothClientCreateDeviceFunc func;
 	gpointer data;
@@ -1448,7 +1592,6 @@ gboolean bluetooth_client_create_device(BluetoothClient *client,
 {
 	CreateDeviceData *devdata;
 	DBusGProxy *adapter;
-	DBusGProxyCall *call;
 
 	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (address != NULL, FALSE);
@@ -1468,17 +1611,17 @@ gboolean bluetooth_client_create_device(BluetoothClient *client,
 	devdata->client = g_object_ref (client);
 
 	if (agent != NULL)
-		call = dbus_g_proxy_begin_call_with_timeout(adapter,
-				"CreatePairedDevice", create_device_callback,
-				devdata, g_free, 90 * 1000,
-				G_TYPE_STRING, address,
-				DBUS_TYPE_G_OBJECT_PATH, agent,
-				G_TYPE_STRING, "DisplayYesNo", G_TYPE_INVALID);
+		dbus_g_proxy_begin_call_with_timeout(adapter,
+						     "CreatePairedDevice", create_device_callback,
+						     devdata, g_free, 90 * 1000,
+						     G_TYPE_STRING, address,
+						     DBUS_TYPE_G_OBJECT_PATH, agent,
+						     G_TYPE_STRING, "DisplayYesNo", G_TYPE_INVALID);
 	else
-		call = dbus_g_proxy_begin_call_with_timeout(adapter,
-				"CreateDevice", create_device_callback,
-				devdata, g_free, 60 * 1000,
-				G_TYPE_STRING, address, G_TYPE_INVALID);
+		dbus_g_proxy_begin_call_with_timeout(adapter,
+						     "CreateDevice", create_device_callback,
+						     devdata, g_free, 60 * 1000,
+						     G_TYPE_STRING, address, G_TYPE_INVALID);
 
 	return TRUE;
 }
@@ -1734,3 +1877,100 @@ gboolean bluetooth_client_disconnect_service (BluetoothClient *client,
 	return TRUE;
 }
 
+#define BOOL_STR(x) (x ? "True" : "False")
+
+static void
+services_foreach (const char *service, gpointer _value, GString *str)
+{
+	gboolean value = GPOINTER_TO_INT (_value);
+	g_string_append_printf (str, "%s (%s) ", service, value ? "connected" : "not connected");
+}
+
+void
+bluetooth_client_dump_device (GtkTreeModel *model,
+			      GtkTreeIter *iter,
+			      gboolean recurse)
+{
+	DBusGProxy *proxy;
+	char *address, *alias, *name, *icon, **uuids;
+	gboolean is_default, paired, trusted, connected, discoverable, discovering, powered, is_adapter;
+	GHashTable *services;
+	GtkTreeIter parent;
+	guint type;
+
+	gtk_tree_model_get (model, iter,
+			    BLUETOOTH_COLUMN_ADDRESS, &address,
+			    BLUETOOTH_COLUMN_ALIAS, &alias,
+			    BLUETOOTH_COLUMN_NAME, &name,
+			    BLUETOOTH_COLUMN_TYPE, &type,
+			    BLUETOOTH_COLUMN_ICON, &icon,
+			    BLUETOOTH_COLUMN_DEFAULT, &is_default,
+			    BLUETOOTH_COLUMN_PAIRED, &paired,
+			    BLUETOOTH_COLUMN_TRUSTED, &trusted,
+			    BLUETOOTH_COLUMN_CONNECTED, &connected,
+			    BLUETOOTH_COLUMN_DISCOVERABLE, &discoverable,
+			    BLUETOOTH_COLUMN_DISCOVERING, &discovering,
+			    BLUETOOTH_COLUMN_POWERED, &powered,
+			    BLUETOOTH_COLUMN_SERVICES, &services,
+			    BLUETOOTH_COLUMN_UUIDS, &uuids,
+			    BLUETOOTH_COLUMN_PROXY, &proxy,
+			    -1);
+	is_adapter = !gtk_tree_model_iter_parent (model, &parent, iter);
+
+	if (is_adapter != FALSE) {
+		/* Adapter */
+		g_print ("Adapter: %s (%s)\n", name, address);
+		if (is_default)
+			g_print ("\tDefault adapter\n");
+		g_print ("\tDiscoverable: %s\n", BOOL_STR (discoverable));
+		if (discovering)
+			g_print ("\tDiscovery in progress\n");
+		g_print ("\t%s\n", powered ? "Is powered" : "Is not powered");
+	} else {
+		/* Device */
+		g_print ("Device: %s (%s)\n", alias, address);
+		g_print ("\tD-Bus Path: %s\n", proxy ? dbus_g_proxy_get_path (proxy) : "(none)");
+		g_print ("\tType: %s Icon: %s\n", bluetooth_type_to_string (type), icon);
+		g_print ("\tPaired: %s Trusted: %s Connected: %s\n", BOOL_STR(paired), BOOL_STR(trusted), BOOL_STR(connected));
+		if (services != NULL) {
+			GString *str;
+
+			str = g_string_new (NULL);
+			g_hash_table_foreach (services, (GHFunc) services_foreach, str);
+			g_print ("\tServices: %s\n", str->str);
+			g_string_free (str, TRUE);
+		}
+		if (uuids != NULL) {
+			guint i;
+			g_print ("\tUUIDs: ");
+			for (i = 0; uuids[i] != NULL; i++)
+				g_print ("%s ", uuids[i]);
+			g_print ("\n");
+		}
+	}
+	g_print ("\n");
+
+	g_free (alias);
+	g_free (address);
+	g_free (icon);
+	if (proxy != NULL)
+		g_object_unref (proxy);
+	if (services != NULL)
+		g_hash_table_unref (services);
+	g_strfreev (uuids);
+
+	if (recurse == FALSE)
+		return;
+
+	if (is_adapter != FALSE) {
+		GtkTreeIter child;
+
+		if (gtk_tree_model_iter_children (model, &child, iter) == FALSE)
+			return;
+		bluetooth_client_dump_device (model, &child, FALSE);
+		while (gtk_tree_model_iter_next (model, &child))
+			bluetooth_client_dump_device (model, &child, FALSE);
+	}
+
+}
+
